Imports System
Imports System.Diagnostics
Imports System.Runtime.InteropServices
Imports System.Text

Public Class ProcessMemory
    Public Enum ProcessAccess As UInteger
        All = &H1F0FFF
        Terminate = &H1
        CreateThread = &H2
        VMOperation = &H8
        VMRead = &H10
        VMWrite = &H20
        DupHandle = &H40
        SetInformation = &H200
        QueryInformation = &H400
        Synchronize = &H100000
    End Enum
    <DllImport("kernel32.dll")> _
    Private Shared Function OpenProcess(ByVal dwDesiredAccess As ProcessAccess, <MarshalAs(UnmanagedType.Bool)> ByVal bInheritHandle As Boolean, ByVal dwProcessId As Integer) As IntPtr
    End Function
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Shared Function CloseHandle(ByVal hObject As IntPtr) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Shared Function ReadProcessMemory( _
    ByVal hProcess As IntPtr, _
    ByVal lpBaseAddress As IntPtr, _
    <Out()> ByVal lpBuffer() As Byte, _
    ByVal dwSize As Integer, _
    ByRef lpNumberOfBytesRead As Integer
  ) As Boolean
    End Function
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Shared Function ReadProcessMemory( _
    ByVal hProcess As IntPtr, _
        ByVal lpBaseAddress As IntPtr, _
       <Out(), MarshalAs(UnmanagedType.AsAny)> ByVal lpBuffer As Object, _
    ByVal dwSize As Integer, _
    ByRef lpNumberOfBytesRead As Integer _
   ) As Boolean
    End Function
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Shared Function ReadProcessMemory( _
    ByVal hProcess As IntPtr, _
    ByVal lpBaseAddress As IntPtr, _
    ByVal lpBuffer As IntPtr, _
    ByVal iSize As Integer, _
    ByRef lpNumberOfBytesRead As Integer) As Boolean
    End Function
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Shared Function WriteProcessMemory(ByVal hProcess As IntPtr, ByVal lpBaseAddress As IntPtr, ByVal lpBuffer As Byte(), ByVal nSize As System.UInt32, <Out()> ByRef lpNumberOfBytesWritten As Int32) As Boolean
    End Function
    Private _Process As Process = Nothing
    Public ReadOnly Property Process() As Process
        Get
            Return _Process
        End Get
    End Property
    Private _Access As UInteger
    Public ReadOnly Property Access() As UInteger
        Get
            Return _Access
        End Get
    End Property
    Private _Handle As IntPtr = IntPtr.Zero
    Public ReadOnly Property Handle() As IntPtr
        Get
            Return _Handle
        End Get
    End Property
    Private _Position As IntPtr = 0
    Public Property Position() As Long
        Get
            Return _Position
        End Get
        Set(ByVal value As Long)
            _Position = value
        End Set
    End Property
    ''' <summary>
    ''' This class allows you to read and write to another processes memory.
    ''' </summary>
    ''' <param name="process">The process to attach to.</param>
    ''' <param name="access">Use ProcessAccess.  Reading requires PROCESS_VM_READ while writing requires both PROCESS_VM_WRITE and PROCESS_VM_OPERATION.</param>
    Public Sub New(ByVal process As Process, ByVal access As UInteger)
        _Process = process
        _Access = access
    End Sub
    Public Sub New(ByVal process As Process, ByVal access As ProcessAccess)
        _Process = process
        _Access = CUInt(access)
    End Sub
    Public Sub Open()
        _Handle = OpenProcess(_Access, False, CUInt(_Process.Id))
    End Sub
    Public Sub Close()
        Dim iRetValue As Integer
        iRetValue = CloseHandle(_Handle)
        If iRetValue = 0 Then
            Throw New Exception("CloseHandle failed")
        End If
    End Sub
#Region " Write Methods "
    Public Function Write(ByVal buffer As Byte()) As Integer
        Return Write(_Position, buffer)
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal buffer As Byte()) As Integer
        Dim BytesWritten As IntPtr
        WriteProcessMemory(_Handle, address, buffer, CUInt(buffer.Length), BytesWritten)
        _Position = address.ToInt64() + BytesWritten.ToInt64()
        Return BytesWritten.ToInt32()
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Byte) As Integer
        Return Write(address, New Byte() {value})
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Single) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Double) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Boolean) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Char) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Short) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As UShort) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Integer) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As UInteger) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As Long) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As ULong) As Integer
        Return Write(address, BitConverter.GetBytes(value))
    End Function
    Public Function Write(ByVal address As IntPtr, ByVal value As String, ByVal encoding As Encoding) As Integer
        Return Write(address, encoding.GetBytes(value))
    End Function
#End Region
#Region " Read Methods "
    Public Function Read(ByVal length As UInteger) As Byte()
        Return Read(_Position, length)
    End Function
    Public Function Read(ByVal address As IntPtr, ByVal length As Integer) As Byte()
        Dim buffer As Byte() = New Byte(length - 1) {}

        Dim _read As IntPtr
        Dim _read2 As Integer
        ReadProcessMemory(_Handle, address, buffer, length, _read2)
        _read = _read2
        _Position = address.ToInt64() + _read.ToInt64()

        Return buffer
    End Function
    Public Function ReadUInt8(ByVal address As IntPtr) As Byte
        Dim temp As Byte() = Read(address, 1)
        Return temp(0)
    End Function
    Public Function ReadSingle(ByVal address As IntPtr) As Single
        Dim read__1 As Single() = New Single(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 4), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(0)
    End Function
    Public Function ReadDouble(ByVal address As IntPtr) As Double
        Dim read__1 As Double() = New Double(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 8), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(0)
    End Function
    Public Function ReadChar(ByVal address As IntPtr) As Char
        Dim read__1 As Char() = New Char(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 2), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(0)
    End Function
    Public Function ReadInt16(ByVal address As IntPtr) As Short
        Dim read__1 As Short() = New Short(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 2), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(1)
    End Function
    Public Function ReadInt32(ByVal address As IntPtr) As Integer
        Dim read__1 As Integer() = New Integer(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 4), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(0)
    End Function
    Public Function ReadUInt32(ByVal address As IntPtr) As UInteger
        Return BitConverter.ToUInt32(Read(address, 4), 0)
    End Function
    Public Function ReadInt64(ByVal address As IntPtr) As Long
        Dim read__1 As Long() = New Long(0) {}
        Dim gch As GCHandle = GCHandle.Alloc(Read(address, 8), GCHandleType.Pinned)
        Marshal.Copy(gch.AddrOfPinnedObject(), read__1, 0, 1)
        gch.Free()
        Return read__1(0)
    End Function
    Public Function ReadString(ByVal address As IntPtr, ByVal length As UInteger, ByVal encoding As Encoding) As String
        Return encoding.GetString(Read(address, length))
    End Function
#End Region

End Class
   
